"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _webpackSources = require("webpack-sources");
var _utils = require("./utils");
var _getI18nKeysFormModules = _interopRequireDefault(require("./utils/getI18nKeysFormModules"));
var _createRegularManifest = require("./utils/createRegularManifest");
var _createMetaManifest = require("./utils/createMetaManifest");
var _applyMetaManifest = require("./utils/applyMetaManifest");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
/**
 * this plugin was creating the chunk specific i18n chunk's.
 */

const pluginName = 'I18nFilesEmitter';
const MODULE_TYPE = 'json/i18n';
const i18ntype = locale => `${MODULE_TYPE}/${locale}`;
class I18nFilesEmitter {
  constructor({
    allI18nObject,
    i18nManifestFileName,
    filenameTemplate,
    locales,
    jsonpFunc,
    i18nMetaManifestFileName,
    entryPointNames = ['main'],
    masterChunkFileName = 'master.i18n.js'
  }) {
    this.locales = locales;
    this.allI18nObject = allI18nObject;
    this.filenameTemplate = filenameTemplate;
    this.jsonpFunc = jsonpFunc;
    this.prevHashes = {};
    this.i18nManifestFileName = i18nManifestFileName;
    this.i18nMetaManifestFileName = i18nMetaManifestFileName || 'i18n-chunk/metaManifest.json';
    this.regularManifest = {};
    this.detailedManifest = {};
    this.entryPointNames = Array.isArray(entryPointNames) ? entryPointNames : [entryPointNames];
    this.masterChunkFileName = masterChunkFileName;
    this.generatedMetaManifest = {
      modules: {},
      masterKeys: []
    };
    this.extraKeys = new Set();
    this.movedKeys = new Set();
    this.entryPointKeys = new Set();
    this.combinedChunks = {};
    this.moduleIdMap = new Map();
    this.moduleToChunks = new Map();
    this.resourceToModule = new Map();
  }
  emitAsset(compilation, filename, source, info) {
    if (!compilation.emitAsset) {
      compilation.assets[filename] = source;
    } else {
      compilation.emitAsset(filename, source, info);
    }
  }
  getI18nObjectByLocale(i18nKeys, locale) {
    const data = {};
    i18nKeys.forEach(key => {
      if (this.allI18nObject[locale] && this.allI18nObject[locale][key] !== undefined) {
        data[key] = this.allI18nObject[locale][key];
      }
    });
    return data;
  }
  getTemplateString(i18nKeys, locale) {
    const i18nObject = this.getI18nObjectByLocale(i18nKeys, locale);
    return `${this.jsonpFunc}(${(0, _utils.jsonToString)(i18nObject)});`;
  }
  renderI18nLocaleChunk(chunk, locale, i18nKeys) {
    const filenameTemplate = this.filenameTemplate.replace(/\[locale\]/g, locale);
    const pathOptions = {
      chunk,
      contentHashType: i18ntype(locale)
    };
    return {
      render: () => new _webpackSources.RawSource(this.getTemplateString(i18nKeys, locale)),
      filenameTemplate,
      pathOptions,
      identifier: `${pluginName}.${chunk.id}.${locale}`,
      hash: chunk.contentHash[i18ntype(locale)]
    };
  }
  captureModuleIds(compilation) {
    console.log('Capturing final module IDs from compilation...');
    this.moduleIdMap.clear();
    this.resourceToModule.clear();
    this.moduleToChunks.clear();

    // Process all modules to build our mapping
    compilation.modules.forEach(module => {
      // Skip modules without resources
      if (!module.resource) return;
      const originalId = module.resource;
      const finalId = module.id;

      // Map resource path to final module ID
      this.moduleIdMap.set(originalId, finalId);
      // Map resource path to module object
      this.resourceToModule.set(originalId, module);

      // Build chunk membership for each module
      const chunkIds = Array.from(module.chunksIterable || []).map(chunk => chunk.id).filter(Boolean);
      if (chunkIds.length > 0) {
        this.moduleToChunks.set(originalId, chunkIds);
      }
    });

    // console.log(`Captured ${this.moduleIdMap.size} module ID mappings`);
  }
  gatherModuleDetails(chunk, i18nKeys) {
    if (!i18nKeys.length) return;
    const stableModuleIds = Array.from(chunk.modulesIterable).map(module => module.resource).filter(Boolean);
    const moduleKey = stableModuleIds.length ? stableModuleIds[0] : `chunk_${chunk.id}`;
    if (!this.detailedManifest[moduleKey]) {
      this.detailedManifest[moduleKey] = {
        chunkId: chunk.id,
        chunkName: chunk.name || `chunk_${chunk.id}`,
        keysUsed: new Set()
      };
    }
    i18nKeys.forEach(key => {
      this.detailedManifest[moduleKey].keysUsed.add(key);
    });
  }
  emitI18nFilesForChunk(chunk, compilation) {
    const i18nKeys = (0, _getI18nKeysFormModules.default)(chunk.modulesIterable);
    const manifestForChunk = {
      assets: {},
      hasI18n: false,
      hasChanges: false
    };
    if (!i18nKeys.length) {
      return manifestForChunk;
    }
    this.gatherModuleDetails(chunk, i18nKeys);
    manifestForChunk.hasI18n = true;
    const isEntryPoint = this.isPartOfEntryPoint(chunk, compilation);
    if (isEntryPoint) {
      i18nKeys.forEach(key => this.entryPointKeys.add(key));
    }
    this.locales.forEach(locale => {
      const result = this.renderI18nLocaleChunk(chunk, locale, i18nKeys);
      const {
        filenameTemplate,
        pathOptions,
        render,
        identifier,
        hash
      } = result;
      const filePath = compilation.getPath(filenameTemplate, pathOptions);
      manifestForChunk.assets[locale] = filePath;
      if (this.prevHashes[identifier] === hash) {
        return;
      }
      manifestForChunk.hasChanges = true;
      this.prevHashes[identifier] = hash;
      const renderedContent = render().source();
      this.emitAsset(compilation, filePath, new _webpackSources.RawSource(renderedContent), {
        locale,
        chunkId: chunk.id,
        chunkName: chunk.name
      });
      if (!this.combinedChunks[locale]) {
        this.combinedChunks[locale] = [];
      }
      this.combinedChunks[locale].push({
        path: filePath,
        content: renderedContent,
        data: this.getI18nObjectByLocale(i18nKeys, locale),
        isEntryPoint: isEntryPoint
      });
    });
    return manifestForChunk;
  }
  isPartOfEntryPoint(chunk, compilation) {
    let isEntryChunk = false;
    this.entryPointNames.forEach(entryName => {
      const entryPoint = compilation.entrypoints.get(entryName);
      if (entryPoint && entryPoint.chunks.includes(chunk)) {
        isEntryChunk = true;
      }
    });
    return isEntryChunk;
  }
  createMetaManifestWithFinalIds(compilation) {
    console.log('Creating meta manifest with final module IDs...');
    const finalMetaManifest = {
      modules: {},
      masterKeys: []
    };
    for (const [resourcePath, moduleData] of Object.entries(this.detailedManifest)) {
      const finalId = this.moduleIdMap.get(resourcePath) || resourcePath;
      const chunkName = moduleData.chunkName || `chunk_${finalId}`;
      const keysArray = Array.isArray(moduleData.keysUsed) ? moduleData.keysUsed : Array.from(moduleData.keysUsed);
      if (keysArray.length > 0) {
        finalMetaManifest.modules[chunkName] = {
          id: moduleData.chunkId,
          finalId: finalId,
          resourcePath: resourcePath,
          keys: keysArray
        };
      }
    }
    this.generatedMetaManifest = finalMetaManifest;
    console.log(`Created meta manifest with ${Object.keys(finalMetaManifest.modules).length} modules`);
  }
  compareWithRemoteManifest(compilation) {
    (0, _applyMetaManifest.applyMetaManifest)(compilation, this);
  }
  emitMasterChunks(compilation) {
    const combinedKeys = new Set([...this.extraKeys, ...this.movedKeys, ...(this.generatedMetaManifest.masterKeys || [])]);

    // console.log(`Extra and moved keys to add to master chunk: ${combinedKeys.size}`);
    // console.log(`Entry point keys: ${this.entryPointKeys.size}`);

    // Add entry point keys to the master keys in meta manifest
    if (this.generatedMetaManifest) {
      this.generatedMetaManifest.masterKeys = [...this.generatedMetaManifest.masterKeys, ...Array.from(this.entryPointKeys)];
    }

    // Create a master object that contains all needed keys
    const allMasterKeys = new Set([...combinedKeys, ...this.entryPointKeys]);

    // console.log(`Creating master chunks with ${allMasterKeys.size} total keys`);

    for (const locale of this.locales) {
      try {
        const masterI18nObject = this.getI18nObjectByLocale([...allMasterKeys], locale);
        console.log(`Master i18n object has ${Object.keys(masterI18nObject).length} keys`);
        const finalContent = `${this.jsonpFunc}(${(0, _utils.jsonToString)(masterI18nObject)});`;
        const outputPath = `i18n-chunk/${locale}/${this.masterChunkFileName}`;
        this.emitAsset(compilation, outputPath, new _webpackSources.RawSource(finalContent), {
          locale,
          chunkName: 'master',
          desc: 'Combined master i18n chunk'
        });
        if (!this.regularManifest.master) {
          this.regularManifest.master = {};
        }
        this.regularManifest.master[locale] = outputPath;
      } catch (error) {
        console.error(`ERROR processing master chunk for locale ${locale}:`, error.message);
        try {
          console.log(`Attempting to emit a fallback master chunk for locale ${locale}`);
          const fallbackI18nObject = this.getI18nObjectByLocale([...combinedKeys], locale);
          const fallbackContent = `${this.jsonpFunc}(${(0, _utils.jsonToString)(fallbackI18nObject)});`;
          const outputPath = `i18n-chunk/${locale}/${this.masterChunkFileName}`;
          this.emitAsset(compilation, outputPath, new _webpackSources.RawSource(fallbackContent), {
            locale,
            chunkName: 'master',
            desc: 'Fallback master i18n chunk (only moved/extra keys)'
          });
          if (!this.regularManifest.master) {
            this.regularManifest.master = {};
          }
          this.regularManifest.master[locale] = outputPath;
          console.log(`Emitted fallback master chunk for locale ${locale}: ${outputPath}`);
        } catch (fallbackError) {
          console.error(`Failed to emit fallback master chunk for locale ${locale}:`, fallbackError.message);
        }
      }
    }
  }
  emitI18nAssert(compilation) {
    console.log('Starting I18n asset emission process...');
    this.captureModuleIds(compilation);
    this.entryPointKeys = new Set();
    compilation.chunks.forEach(chunk => {
      const manifestForChunk = this.emitI18nFilesForChunk(chunk, compilation);
      if (manifestForChunk.hasI18n) {
        this.regularManifest[chunk.id] = manifestForChunk.assets;
      }
    });
    this.createMetaManifestWithFinalIds(compilation);
    this.compareWithRemoteManifest(compilation);
    this.emitMasterChunks(compilation);
    (0, _createRegularManifest.createRegularManifest)(compilation, this);
    const metaManifestSource = new _webpackSources.RawSource(JSON.stringify(this.generatedMetaManifest, null, 2));
    this.emitAsset(compilation, this.i18nMetaManifestFileName, metaManifestSource, {
      desc: 'Updated meta manifest with master keys'
    });
    console.log('Completed I18n asset emission process');
  }
  apply(compiler) {
    compiler.hooks.afterCompile.tap(pluginName, compilation => {
      this.compilation = compilation;
    });
    compiler.hooks.emit.tapAsync(pluginName, (compilation, callback) => {
      this.emitI18nAssert(compilation);
      callback();
    });
  }
}
exports.default = I18nFilesEmitter;