"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMetaManifest = createMetaManifest;
var _webpackSources = require("webpack-sources");
function createMetaManifest(compilation, plugin, skipEmit = false) {
  const {
    detailedManifest
  } = plugin;
  const i18nMetaManifestFileName = plugin.i18nMetaManifestFileName;
  if (!skipEmit) {
    console.log('Generating meta manifest:', i18nMetaManifestFileName);
  }
  const finalMetaManifest = {
    modules: {}
  };
  for (const [chunkId, chunkData] of Object.entries(detailedManifest)) {
    const chunkName = chunkData.chunkName || `chunk_${chunkId}`;
    const keysArray = Array.isArray(chunkData.keysUsed) ? chunkData.keysUsed : Array.from(chunkData.keysUsed);
    if (keysArray.length > 0) {
      finalMetaManifest.modules[chunkName] = {
        id: chunkId,
        keys: keysArray
      };
    }
  }
  if (plugin.extraKeys && plugin.movedKeys) {
    const masterKeysSet = new Set([...(Array.from(plugin.extraKeys) || []), ...(Array.from(plugin.movedKeys) || [])]);
    finalMetaManifest.masterKeys = Array.from(masterKeysSet);
    if (finalMetaManifest.masterKeys.length > 0) {
      console.log(`Extra and moved keys to add to master chunk: ${finalMetaManifest.masterKeys.length}`);
      finalMetaManifest.masterKeys.forEach(key => {
        console.log(`  - ${key}`);
      });
    }
  }
  plugin.generatedMetaManifest = finalMetaManifest;
  if (!skipEmit) {
    const metaManifestSource = new _webpackSources.RawSource(JSON.stringify(finalMetaManifest, null, 2));
    if (!compilation.emitAsset) {
      compilation.assets[i18nMetaManifestFileName] = metaManifestSource;
    } else {
      compilation.emitAsset(i18nMetaManifestFileName, metaManifestSource, {
        desc: 'Streamlined chunk-module-i18nKeys mapping with optimized master keys'
      });
    }
  }
  return finalMetaManifest;
}