"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRegularManifest = createRegularManifest;
var _webpackSources = require("webpack-sources");
function createRegularManifest(compilation, plugin) {
  const {
    regularManifest,
    // e.g., { chunkId: { localeA: pathA, localeB: pathB, ... }, ... }
    locales,
    // e.g., ['en_US', 'fr_FR', ...]
    i18nManifestFileName,
    // e.g., 'i18n-manifest.json'
    deskI18nManifestFileName = 'i18n-chunk/desk-i18n-manifest.js'
  } = plugin;
  const finalManifest = {
    locales: [...locales],
    chunks: {},
    basePath: 'i18n-chunk/[locale]/'
  };
  for (const chunkId of Object.keys(regularManifest)) {
    const localePaths = regularManifest[chunkId];
    if (!localePaths) continue;
    const entries = Object.entries(localePaths);
    if (!entries.length) continue;
    const [someLocale, fullPath] = entries[0];
    if (!fullPath) continue;
    const pathSegments = fullPath.split('/');
    const basename = pathSegments.slice(2).join('/');
    finalManifest.chunks[chunkId] = basename;
  }
  const manifestStr = JSON.stringify(finalManifest, null, 2);
  const jsonManifestSource = new _webpackSources.RawSource(manifestStr);
  const deskI18nManifestCode = `
    (function() {
      window.desk_i18n_manifest = ${manifestStr};
    })();
  `;
  const deskI18nManifestSource = new _webpackSources.RawSource(deskI18nManifestCode);
  compilation.emitAsset(i18nManifestFileName, jsonManifestSource, {
    desc: 'Emitted plain JSON i18n manifest'
  });
  compilation.emitAsset(deskI18nManifestFileName, deskI18nManifestSource, {
    desc: 'Emitted manifest attached to window.desk_i18n_manifest'
  });
}