"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

// // plugins/I18nPlugin.js
// const path = require('path');
// const fs = require('fs');
// const getJsResourceKeys = require('./utils/getJsResourceKeys');
// const { collectI18nKeysfromAST, collectI18nKeysfromComments } = require('../I18nSplitPlugin/utils/collectI18nKeys');
// const NullFactory = require('webpack/lib/NullFactory');
// const { I18nDependency, I18nDependencyTemplate } = require('../I18nSplitPlugin/I18nDependency');
// class I18nPlugin {
//   constructor(options = {}) {
//     const { jsResourcePath } = options;
//     if (!jsResourcePath) {
//       throw new Error('I18nPlugin: "jsResourcePath" option is required.');
//     }
//     this.jsResourcePath = jsResourcePath;
//     this.jsresourceKeys = getJsResourceKeys(jsResourcePath);
//     this.moduleToKeysMap = new Map();
//   }
// identifyEntryPoints(compilation) {
//     this.entryPoints = new Set();
//     const entryModules = new Set();
//     compilation.entrypoints.forEach((entrypoint) => {
//       entrypoint.getEntrypointChunk().getModules().forEach((module) => {
//         if (module.resource) {
//           entryModules.add(module.resource);
//           console.log(entryModules)
//         }
//       });
//     });
//     this.entryPoints = entryModules;
//   }
//   apply(compiler) {
//     compiler.hooks.thisCompilation.tap('I18nPlugin', (compilation, params) => {
//       // Define dependency factories and templates for I18nDependency
//       this.defineI18nDependency(compilation);
//       // Handler for parsing modules
//       const handler = (parser) => {
//         parser.hooks.program.tap('I18nPlugin', (ast, comments) => {
//           const { module } = parser.state;
//           if (!module || !module.resource || !/\.(js|jsx)$/.test(module.resource)) {
//             return;
//           }
//           const moduleResource = module.resource;
//           // Collect i18n keys from AST and comments
//           let i18nKeys = collectI18nKeysfromAST(ast, this.jsresourceKeys);
//           i18nKeys = i18nKeys.concat(collectI18nKeysfromComments(comments, this.jsresourceKeys));
//           if (i18nKeys.length > 0) {
//             // Store the keys for this module
//             this.moduleToKeysMap.set(moduleResource, i18nKeys);
//             // Add I18nDependency to the module
//             this.addI18nDependency(module, i18nKeys);
//             console.log(`Module: ${moduleResource}`);
//          //   console.log(`I18n Keys: ${i18nKeys.join(', ')}`);
//           }
//         });
//       };
//       // Attach the handler to JavaScript parsers
//       const normalModuleFactory = params.normalModuleFactory;
//       normalModuleFactory.hooks.parser.for('javascript/auto').tap('I18nPlugin', handler);
//       normalModuleFactory.hooks.parser.for('javascript/dynamic').tap('I18nPlugin', handler);
//       normalModuleFactory.hooks.parser.for('javascript/esm').tap('I18nPlugin', handler);
//       // After modules are built, process the collected keys
//       compilation.hooks.finishModules.tap('I18nPlugin', (modules) => {
//         this.identifyEntryPoints(compilation);
//         this.processCollectedKeys(compilation);
//       });
//     });
//   }
//   defineI18nDependency(compilation) {
//     compilation.dependencyFactories.set(I18nDependency, new NullFactory());
//     compilation.dependencyTemplates.set(I18nDependency, new I18nDependencyTemplate());
//   }
//   addI18nDependency(module, i18nKeys) {
//     const identifier = `i18n ${module.resource}`;
//     const dep = new I18nDependency({ identifier, i18nKeys }, module.context, 0);
//     module.addDependency(dep);
//   }
//   processCollectedKeys(compilation) {
//     // Implement logic to process collected keys
//     // For now, we'll just console log all the modules and their keys
//     for (const [moduleResource, keys] of this.moduleToKeysMap.entries()) {
//      // console.log(`Module: ${moduleResource} has keys: ${keys.join(', ')}`);
//     }
//     // Next steps:
//     // - Identify entry point modules
//     // - Assign keys to chunks (Chunk Zero for entry points, last chunk for unused keys)
//     // - Implement the chunking logic as per your requirements
//   }
// }
// module.exports = I18nPlugin;
// plugins/I18nPlugin.js
const path = require('path');
const fs = require('fs');
const getPropertiesAsJSON = require('./utils/getJsResourceKeys');
const {
  collectI18nKeysfromAST,
  collectI18nKeysfromComments
} = require('../I18nSplitPlugin/utils/collectI18nKeys');
const NullFactory = require('webpack/lib/NullFactory');
const {
  I18nDependency,
  I18nDependencyTemplate
} = require('../I18nSplitPlugin/I18nDependency');
const pluginName = 'I18nPlugin';
class I18nPlugin {
  constructor(options = {}) {
    const {
      jsResourcePath,
      mainChunkName
    } = options;
    if (!jsResourcePath) {
      throw new Error('I18nPlugin: "jsResourcePath" option is required.');
    }
    if (!mainChunkName) {
      throw new Error('I18nPlugin: "mainChunkName" option is required.');
    }
    this.jsResourcePath = jsResourcePath;
    this.entryPointName = mainChunkName;
    this.allI18nKeys = getPropertiesAsJSON(this.jsResourcePath);
    console.log("allI18nKeys ", this.allI18nKeys.size);
    this.moduleToKeysMap = new Map();
    this.keyToModulesMap = new Map();
  }
  defineI18nDependency(compilation) {
    compilation.dependencyFactories.set(I18nDependency, new NullFactory());
    compilation.dependencyTemplates.set(I18nDependency, new I18nDependencyTemplate());
  }
  addI18nDependency(module, i18nKeys) {
    const identifier = `i18n ${module.resource}`;
    const dep = new I18nDependency({
      identifier,
      i18nKeys
    }, module.context, 0);
    module.addDependency(dep);
  }
  apply(compiler) {
    compiler.hooks.thisCompilation.tap(pluginName, (compilation, params) => {
      this.defineI18nDependency(compilation); //   const handler = parser => {
      //     parser.hooks.program.tap(pluginName, (ast, commmets) => {
      //       const { module } = parser.state;
      //       //
      //       if (!(module && /\.jsx?$/.test(module.resource))) {
      //         return;
      //       }
      //       // i18n grep proper contract regex
      //       let i18nKeys = collectI18nKeysfromAST(ast, this.allI18nKeys );
      //       i18nKeys = i18nKeys.concat(
      //        //  i18n grep proper contract regex
      //         collectI18nKeysfromComments(commmets, this.allI18nKeys)
      //       );
      //       if (i18nKeys.length) {
      //         console.log(i18nKeys)
      //         this.addI18nDependency(module, i18nKeys);
      //       }
      //     });
      //   };

      const handler = parser => {
        // Initialize counters for each type of log
        let moduleLogCount = 0;
        let astKeysLogCount = 0;
        let commentKeysLogCount = 0;
        let combinedKeysLogCount = 0;
        let dependencyLogCount = 0;
        parser.hooks.program.tap(pluginName, (ast, comments) => {
          const {
            module
          } = parser.state;
          if (moduleLogCount < 20) {
            console.log("Processing module:", module ? module.resource : "No module found");
            moduleLogCount++;
          } // Check if the module and its resource match the .js or .jsx regex

          if (!(module && /\.jsx?$/.test(module.resource))) {
            if (moduleLogCount < 20) {
              console.warn("Skipping non-JS/JSX module or missing module resource.");
              moduleLogCount++;
            }
            return;
          }
          try {
            // Collect i18n keys from AST
            let i18nKeys = collectI18nKeysfromAST(ast, this.allI18nKeys);
            if (astKeysLogCount < 20) {
              console.log("i18n keys from AST:", i18nKeys);
              astKeysLogCount++;
            } // Collect i18n keys from comments

            const commentKeys = collectI18nKeysfromComments(comments, this.allI18nKeys);
            if (commentKeysLogCount < 20) {
              console.log("i18n keys from comments:", commentKeys);
              commentKeysLogCount++;
            } // Combine keys from AST and comments

            i18nKeys = i18nKeys.concat(commentKeys);
            if (combinedKeysLogCount < 20) {
              console.log("Combined i18n keys:", i18nKeys);
              combinedKeysLogCount++;
            } // If we have collected any keys, add them as dependencies

            if (i18nKeys.length) {
              this.addI18nDependency(module, i18nKeys);
              if (dependencyLogCount < 20) {
                console.log(`Added ${i18nKeys.length} i18n keys as dependencies.`);
                dependencyLogCount++;
              }
            } else {
              if (dependencyLogCount < 20) {
                console.warn("No i18n keys found to add as dependencies.");
                dependencyLogCount++;
              }
            }
          } catch (error) {
            console.error("Error processing module:", module.resource, error);
          }
        });
      };
      const normalModuleFactory = params.normalModuleFactory;
      normalModuleFactory.hooks.parser.for('javascript/auto').tap(pluginName, handler);
      normalModuleFactory.hooks.parser.for('javascript/dynamic').tap(pluginName, handler);
      normalModuleFactory.hooks.parser.for('javascript/esm').tap(pluginName, handler);
      compilation.hooks.afterOptimizeChunks.tap(pluginName, () => {
        this.identifyEntryPoints(compilation); // this.processCollectedKeys(compilation);
      });
    });
  }
  identifyEntryPoints(compilation) {
    this.entryPoints = new Set();
    const entryModules = new Set();
    console.log("Starting identifyEntryPoints..."); // Iterate over each entrypoint in the compilation

    compilation.entrypoints.forEach((entrypoint, name) => {
      console.log(`Processing entrypoint: ${name}`); // Logs the entrypoint name

      if (name === "main") {
        console.log(`Found the "main" entrypoint.`); // Debug log if "main" is found
      } // Process all chunks associated with the entrypoint

      entrypoint.chunks.forEach(chunk => {
        //  console.log(`Processing chunk: ${chunk.name}`);
        chunk.getModules().forEach(module => {
          if (module.resource) {
            entryModules.add(module.resource); // console.log(`Added module: ${module.resource}`);
          }
        });
      });
    });
    this.entryPoints = entryModules; // console.log("Entry Points Identified:", Array.from(this.entryPoints));
  }
  processCollectedKeys(compilation) {
    // Map keys to chunk IDs
    this.keyToChunkIdMap = {};
    const unusedKeys = new Set(); // Assign keys used in entry points to Chunk Zero

    for (const moduleResource of this.entryPoints) {
      const keys = this.moduleToKeysMap.get(moduleResource);
      if (keys) {
        for (const key of keys) {
          this.keyToChunkIdMap[key] = 0; // Assign to Chunk Zero
        }
      }
    } // Assign other keys to their respective modules

    let chunkIdCounter = 1; // Start from 1 since 0 is for entry points

    for (const [key, modules] of this.keyToModulesMap.entries()) {
      if (!this.keyToChunkIdMap.hasOwnProperty(key)) {
        this.keyToChunkIdMap[key] = chunkIdCounter++;
      }
    } // Collect unused keys (keys not found in jsresource.properties)

    for (const key of Object.keys(this.allI18nKeys)) {
      if (!this.keyToModulesMap.has(key)) {
        unusedKeys.add(key);
      }
    } // Assign unused keys to the last chunk

    const lastChunkId = chunkIdCounter;
    for (const key of unusedKeys) {
      this.keyToChunkIdMap[key] = lastChunkId;
    } // Store total number of chunks

    this.totalChunks = lastChunkId + 1; // Output key-to-chunk mapping for use in i18n build

    const outputPath = compilation.options.output.path;
    const mappingFilePath = path.join(outputPath, 'i18n-key-chunk-mapping.json');
    fs.writeFileSync(mappingFilePath, JSON.stringify(this.keyToChunkIdMap, null, 2), 'utf8');
  }
}
exports.default = I18nPlugin;
I18nPlugin.isChunkHasI18n = function (chunk) {
  for (const module of chunk.modulesIterable) {
    for (const dep of module.dependencies) {
      if (dep instanceof I18nDependency) {
        return true;
      }
    }
  }
  return false;
};