"use strict";

// GenerateModuleIdToKeysMapPlugin.js (Updated)
const fs = require('fs');
const path = require('path');
class GenerateModuleIdToKeysMapPlugin {
  constructor(options = {}) {
    this.moduleToKeysMapPath = options.moduleToKeysMapPath || 'module_to_keys_map.json';
    this.outputPath = options.outputPath || 'module_id_to_keys_map.json';
  }
  apply(compiler) {
    compiler.hooks.emit.tapAsync('GenerateModuleIdToKeysMapPlugin', (compilation, callback) => {
      console.log('[GenerateModuleIdToKeysMapPlugin] Plugin is running.'); // Load the existing module_to_keys_map.json

      let moduleToKeysMap;
      try {
        const data = fs.readFileSync(this.moduleToKeysMapPath, 'utf8');
        moduleToKeysMap = JSON.parse(data); // console.log('[GenerateModuleIdToKeysMapPlugin] moduleToKeysMap loaded.');
      } catch (error) {
        // console.error(`[GenerateModuleIdToKeysMapPlugin] Error reading ${this.moduleToKeysMapPath}:`, error);
        moduleToKeysMap = {};
      }
      const projectRoot = path.resolve(__dirname, '..'); // Adjust '..' as needed
      // Build a mapping from module relative paths to module IDs

      const modulePathToIdMap = new Map();
      for (const module of compilation.modules) {
        if (module.resource) {
          const moduleResource = module.resource;
          const relativeModuleResource = path.relative(projectRoot, moduleResource).replace(/\\/g, '/').toLowerCase();
          const moduleId = module.id;
          modulePathToIdMap.set(relativeModuleResource, moduleId);
        }
      } //  console.log('[GenerateModuleIdToKeysMapPlugin] Available module paths and IDs:');

      for (const [modulePath, moduleId] of modulePathToIdMap.entries()) {//  console.log(`Module Path: ${modulePath}, Module ID: ${moduleId}`);
      } // Transform module paths in moduleToKeysMap to module IDs

      const moduleIdToKeysMap = {};
      for (const [modulePath, keysData] of Object.entries(moduleToKeysMap)) {
        const normalizedModulePath = modulePath.replace(/\\/g, '/').toLowerCase();
        const moduleId = modulePathToIdMap.get(normalizedModulePath);
        if (moduleId == null) {
          // console.warn(`[GenerateModuleIdToKeysMapPlugin] Module ID not found for path: ${normalizedModulePath}`);
          continue;
        }
        const moduleIdString = String(moduleId);
        moduleIdToKeysMap[moduleIdString] = keysData;
      } // Write the module_id_to_keys_map.json

      try {
        fs.writeFileSync(this.outputPath, JSON.stringify(moduleIdToKeysMap, null, 2), 'utf8'); //   console.log(`[GenerateModuleIdToKeysMapPlugin] Generated ${this.outputPath}`);
      } catch (error) {//  console.error(`[GenerateModuleIdToKeysMapPlugin] Error writing ${this.outputPath}:`, error);
      }
      callback();
    });
  }
}
module.exports = GenerateModuleIdToKeysMapPlugin;