"use strict";

// const { RawSource } = require('webpack-sources');
// const fs = require('fs');
// const path = require('path');
// //const axios = require('axios');
// class I18nDiffPlugin {
//   constructor(options = {}) {
//     const {
//       localMetaManifestPath,
//       liveMetaManifestUrl,
//       i18nAssetsOutputPath,
//       i18nDataPath,
//       locales = [],
//     } = options;
//     if (!localMetaManifestPath || !liveMetaManifestUrl || !i18nDataPath) {
//       throw new Error('I18nDiffPlugin: Missing required options.');
//     }
//     this.localMetaManifestPath = localMetaManifestPath;
//     this.liveMetaManifestUrl = liveMetaManifestUrl;
//     this.i18nAssetsOutputPath = i18nAssetsOutputPath || 'dist/i18n-diff-chunks';
//     this.i18nDataPath = i18nDataPath;
//     this.locales = locales;
//   }
//   apply(compiler) {
//     compiler.hooks.emit.tapAsync('I18nDiffPlugin', async (compilation, callback) => {
//       try {
//         // Step 1: Fetch the live CDN meta manifest
//       //  const liveManifest = await this.fetchLiveManifest();
//       const liveManifest = fs.readFileSync("./metaManifest.json", 'utf-8');
//         // Step 2: Read the local meta manifest
//         const localManifest = this.readLocalManifest();
//         // Step 3: Compare manifests to find new or moved keys
//         const diffKeys = this.compareManifests(localManifest, liveManifest);
//         if (diffKeys.length === 0) {
//           console.log('I18nDiffPlugin: No new or moved i18n keys detected.');
//           return callback();
//         }
//         console.log(`I18nDiffPlugin: Found ${diffKeys.length} new or moved i18n keys.`);
//         // Step 4: Generate i18n chunks for the diff keys
//         await this.generateI18nChunks(compilation, diffKeys);
//         // Step 5: Include chunks in build output (handled by emitting assets)
//         callback();
//       } catch (error) {
//         console.error('I18nDiffPlugin: Error during processing:', error);
//         callback(error);
//       }
//     });
//   }
// //   async fetchLiveManifest() {
// //     console.log('I18nDiffPlugin: Fetching live CDN meta manifest...');
// //     const response = await axios.get(this.liveMetaManifestUrl);
// //     return response.data;
// //   }
//   readLocalManifest() {
//     console.log('I18nDiffPlugin: Reading local meta manifest...');
//     const manifestContent = fs.readFileSync(this.localMetaManifestPath, 'utf-8');
//     return JSON.parse(manifestContent);
//   }
//   compareManifests(localManifest, liveManifest) {
//     const localKeys = new Set(Object.keys(localManifest));
//     const liveKeys = new Set(Object.keys(liveManifest));
//     // Identify keys that are in local but not in live (new or moved keys)
//     const diffKeys = [...localKeys].filter((key) => !liveKeys.has(key));
//     return diffKeys;
//   }
//   async generateI18nChunks(compilation, diffKeys) {
//     console.log('I18nDiffPlugin: Generating i18n chunks for new or moved keys...');
//     for (const locale of this.locales) {
//       const localeData = await this.loadLocaleData(locale);
//       const i18nData = {};
//       for (const key of diffKeys) {
//         if (localeData[key]) {
//           i18nData[key] = localeData[key];
//         } else {
//           console.warn(`I18nDiffPlugin: Key "${key}" not found in locale "${locale}" data.`);
//         }
//       }
//       const chunkContent = `window.loadI18nChunk(${JSON.stringify(i18nData)});`;
//       const chunkFilename = `${this.i18nAssetsOutputPath}/${locale}/i18n-diff-chunk.js`;
//       // Ensure the path exists
//       const outputPath = path.dirname(chunkFilename);
//       if (!fs.existsSync(outputPath)) {
//         fs.mkdirSync(outputPath, { recursive: true });
//       }
//       // Emit the chunk
//       compilation.assets[chunkFilename] = new RawSource(chunkContent);
//     }
//   }
//   async loadLocaleData(locale) {
//     const localeFilePath = path.join(this.i18nDataPath, `${locale}.json`);
//     if (!fs.existsSync(localeFilePath)) {
//       throw new Error(`I18nDiffPlugin: Locale data file not found for "${locale}" at "${localeFilePath}".`);
//     }
//     const content = fs.readFileSync(localeFilePath, 'utf-8');
//     return JSON.parse(content);
//   }
// }
// module.exports = I18nDiffPlugin;
const {
  RawSource
} = require('webpack-sources');
const fs = require('fs');
const path = require('path'); //const getI18nKeysFromModules = require('./utils/getI18nKeysFromModules');

const {
  convertUnicode
} = require('../I18nSplitPlugin/utils/unicodeConversion');
const pluginName = 'I18nDiffPlugin';
class I18nDiffPlugin {
  constructor(options = {}) {
    const {
      localMetaManifestPath,
      liveMetaManifestPath,
      i18nAssetsOutputPath,
      i18nDataPath,
      jsonpFunc
    } = options;
    if (!localMetaManifestPath || !liveMetaManifestPath || !i18nDataPath) {
      throw new Error('I18nDiffPlugin: Missing required options.');
    }
    this.localMetaManifestPath = localMetaManifestPath;
    this.liveMetaManifestPath = liveMetaManifestPath;
    this.i18nAssetsOutputPath = i18nAssetsOutputPath || 'dist/i18n-diff-chunks';
    this.i18nDataPath = i18nDataPath;
    this.jsonpFunc = jsonpFunc || 'window.loadI18nChunk';
    this.allI18nObject = {};
    this.locales = [];
  }
  apply(compiler) {
    compiler.hooks.emit.tapAsync(pluginName, (compilation, callback) => {
      try {
        // Step 1: Load the live CDN meta manifest
        // const liveManifest = this.loadManifest(this.liveMetaManifestPath);
        const liveManifest = this.loadManifest("./metaManifest.json"); // Step 2: Read the local meta manifest

        const localManifest = this.loadManifest(this.localMetaManifestPath); // Step 3: Compare manifests to find new or moved keys

        const {
          newKeys,
          movedKeys
        } = this.compareManifests(localManifest, liveManifest);
        const diffKeys = [...new Set([...newKeys, ...movedKeys])];
        if (diffKeys.length === 0) {
          console.log('I18nDiffPlugin: No new or moved i18n keys detected.');
          return callback();
        }
        console.log(`I18nDiffPlugin: Found ${diffKeys.length} new or moved i18n keys.`);
        console.log(`New keys: ${newKeys.join(', ')}`);
        console.log(`Moved keys: ${movedKeys.join(', ')}`); // Step 4: Load all i18n data

        this.loadAllI18nData(); // Step 5: Generate i18n chunks for the diff keys

        this.generateI18nChunks(compilation, diffKeys);
        callback();
      } catch (error) {
        console.error('I18nDiffPlugin: Error during processing:', error);
        callback(error);
      }
    });
  }
  loadManifest(manifestPath) {
    console.log(`I18nDiffPlugin: Loading manifest from ${manifestPath}...`);
    const manifestContent = fs.readFileSync(manifestPath, 'utf-8');
    return JSON.parse(manifestContent);
  }
  compareManifests(localManifest, liveManifest) {
    const localKeys = Object.keys(localManifest.keyToCacheGroupMap);
    const liveKeys = Object.keys(liveManifest.keyToCacheGroupMap);
    const newKeys = localKeys.filter(key => !liveKeys.includes(key));
    const movedKeys = localKeys.filter(key => {
      if (liveKeys.includes(key)) {
        const localGroups = localManifest.keyToCacheGroupMap[key];
        const liveGroups = liveManifest.keyToCacheGroupMap[key];
        return JSON.stringify(localGroups) !== JSON.stringify(liveGroups);
      }
      return false;
    });
    return {
      newKeys,
      movedKeys
    };
  } //   loadAllI18nData() {
  //     console.log('I18nDiffPlugin: Loading all i18n data...');
  //     const files = fs.readdirSync(this.i18nDataPath);
  //     files.forEach((file) => {
  //       const locale = path.basename(file, path.extname(file));
  //       const localeFilePath = path.join(this.i18nDataPath, file);
  //       const content = fs.readFileSync(localeFilePath, 'utf-8');
  //       this.allI18nObject[locale] = JSON.parse(content);
  //       this.locales.push(locale);
  //     });
  //     console.log(`I18nDiffPlugin: Locales found: ${this.locales.join(', ')}`);
  //   }

  loadAllI18nData() {
    console.log('I18nDiffPlugin: Loading all i18n data...');
    const files = fs.readdirSync(this.i18nDataPath);
    files.forEach(file => {
      const locale = path.basename(file, path.extname(file));
      const localeFilePath = path.join(this.i18nDataPath, file);
      const content = fs.readFileSync(localeFilePath, 'utf-8'); // Regular expression to match 'var i18n = {...};'

      const regex = /var\s+i18n\s*=\s*(\{[\s\S]*\});?/;
      const match = content.match(regex);
      if (match && match[1]) {
        const jsonString = match[1];
        try {
          const i18nData = eval('(' + jsonString + ')'); // Use eval carefully

          this.allI18nObject[locale] = i18nData;
          this.locales.push(locale);
        } catch (err) {
          console.error(`I18nDiffPlugin: Failed to parse i18n data in ${file}:`, err);
        }
      } else {
        console.warn(`I18nDiffPlugin: Could not extract 'i18n' variable from ${file}`);
      }
    });
    console.log(`I18nDiffPlugin: Locales found: ${this.locales.join(', ')}`);
  }
  generateI18nChunks(compilation, diffKeys) {
    console.log('I18nDiffPlugin: Generating i18n chunks for new or moved keys...');
    const chunkName = 'i18n-diff-chunk'; // For each locale, generate the i18n chunk

    this.locales.forEach(locale => {
      const i18nData = this.getI18nObjectByLocale(diffKeys, locale);
      const moduleSource = this.getTemplateString(i18nData);
      const filename = path.join(this.i18nAssetsOutputPath, locale, `${chunkName}.js`); // Ensure the path exists

      const outputPath = path.dirname(filename);
      if (!fs.existsSync(outputPath)) {
        fs.mkdirSync(outputPath, {
          recursive: true
        });
      } // Emit the chunk

      compilation.assets[filename] = new RawSource(moduleSource);
    });
  }
  getI18nObjectByLocale(i18nKeys, locale) {
    const data = {};
    i18nKeys.forEach(key => {
      if (this.allI18nObject[locale][key]) {
        data[key] = this.allI18nObject[locale][key];
      } else {
        console.warn(`I18nDiffPlugin: Key "${key}" not found in locale "${locale}" data.`);
      }
    });
    return data;
  }
  getTemplateString(i18nObject) {
    // const convertedI18nObject = convertUnicode(
    //   JSON.stringify(i18nObject, null, 0)
    // );
    const jsonString = JSON.stringify(i18nObject); //console.log('i18nObject:', i18nObject);

    console.log('jsonString:', jsonString);
    return `${this.jsonpFunc}(${jsonString});`;
  }
}
module.exports = I18nDiffPlugin;