"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _webpack = require("webpack");
var _path = _interopRequireDefault(require("path"));
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

// import { Template } from 'webpack';
// const pluginName = 'I18nDownloadLogic';
// export default class I18nDownloadLogic {
//   constructor({ localeVarName = 'document.documentElement.lang', publicPath = '' }) {
//     this.options = {
//       localeVarName,
//       publicPath: publicPath.endsWith('/') ? publicPath : publicPath + '/'
//     };
//   }
//   apply(compiler) {
//     compiler.hooks.thisCompilation.tap(pluginName, compilation => {
//       const { mainTemplate } = compilation;
//       mainTemplate.hooks.requireEnsure.tap(pluginName, (source, chunk, hash) => {
//         return Template.asString([
//           source.replace(
//             'var promise = new Promise(function(resolve, reject) {',
//             'var originalPromise = new Promise(function(resolve, reject) {'
//           ),
//           '',
//           '// I18n chunk loading logic',
//           '(function(chunkId, promises) {',
//           Template.indent([
//             `var locale = ${this.options.localeVarName};`,
//             'if (!__webpack_require__.i18n) {',
//             Template.indent([
//               '__webpack_require__.i18n = {',
//               '  loaded: {},',
//               '  manifest: null,',
//               '  manifestLoading: null,',
//               '  loadManifest: function() {',
//               '    var self = this;',
//               '    if (this.manifest) return Promise.resolve(this.manifest);',
//               '    if (this.manifestLoading) return this.manifestLoading;',
//               '',
//               '    console.log("[I18N] Fetching manifest...");',
//               '    this.manifestLoading = fetch(__webpack_require__.p + "i18n-manifest.json")',
//               '      .then(function(res) {',
//               '        if (!res.ok) throw new Error("Failed to load manifest: " + res.status);',
//               '        return res.json();',
//               '      })',
//               '      .then(function(data) {',
//               '        console.log("[I18N] Manifest loaded:", data);',
//               '        self.manifest = data;',
//               '        self.manifestLoading = null;',
//               '        return data;',
//               '      })',
//               '      .catch(function(error) {',
//               '        console.error("[I18N] Manifest load failed:", error);',
//               '        self.manifestLoading = null;',
//               '        throw error;',
//               '      });',
//               '',
//               '    return this.manifestLoading;',
//               '  },',
//               '  getChunkI18nFile: function(chunkId, locale) {',
//               '    try {',
//               '      if (!this.manifest || !this.manifest.locales) return null;',
//               '      var localeData = this.manifest.locales[locale];',
//               '      return localeData && localeData[chunkId];',
//               '    } catch (e) {',
//               '      console.error("[I18N] Error getting chunk file:", e);',
//               '      return null;',
//               '    }',
//               '  },',
//               '  loadChunkI18n: function(chunkId, locale) {',
//               '    var self = this;',
//               '    var key = chunkId + "_" + locale;',
//               '',
//               '    if (this.loaded[key]) {',
//               '      return Promise.resolve();',
//               '    }',
//               '',
//               '    return new Promise(function(resolve, reject) {',
//               '      var i18nFile = self.getChunkI18nFile(chunkId, locale);',
//               '      if (!i18nFile) {',
//               '        // Not an error - this chunk might not have i18n data',
//               '        console.log("[I18N] No i18n file for chunk:", chunkId);',
//               '        resolve();',
//               '        return;',
//               '      }',
//               '',
//               '      var script = document.createElement("script");',
//               '      script.charset = "utf-8";',
//               '      script.setAttribute("data-chunk-id", chunkId);',
//               '',
//               '      script.onload = function() {',
//               '        self.loaded[key] = true;',
//               '        resolve();',
//               '      };',
//               '',
//               '      script.onerror = function(error) {',
//               '        console.error("[I18N] Failed to load chunk i18n:", chunkId, error);',
//               '        // Don\'t reject - we don\'t want to block chunk loading',
//               '        resolve();',
//               '      };',
//               '',
//               '      script.src = __webpack_require__.p + i18nFile;',
//               '      document.head.appendChild(script);',
//               '    });',
//               '  }',
//               '};'
//             ]),
//             '}',
//             '',
//             '// Load i18n data for this chunk if needed',
//             'if (promises && Array.isArray(promises)) {',
//             '  promises.push(',
//             '    __webpack_require__.i18n.loadManifest()',
//             '      .then(function() {',
//             '        return __webpack_require__.i18n.loadChunkI18n(chunkId, locale);',
//             '      })',
//             '      .catch(function(error) {',
//             '        console.warn("[I18N] Failed to load i18n for chunk:", chunkId, error);',
//             '      })',
//             '  );',
//             '}'
//           ]),
//           '})(__webpack_require__.u ? chunkId : (typeof chunkId === "number" ? "" + chunkId : chunkId), promises);'
//         ]);
//       });
//     });
//   }
// }
const pluginName = 'I18nDownloadLogic';
class I18nDownloadLogic {
  constructor({
    filenameTemplate = 'i18n-chunk/[locale]/[name].[chunkhash].i18n.js',
    publicPath = '',
    localeVarName = 'document.documentElement.lang',
    locales = []
  }) {
    this.locales = locales;
    this.publicPath = publicPath;
    this.options = {
      filenameTemplate,
      localeVarName
    };
  }
  apply(compiler) {
    compiler.hooks.thisCompilation.tap(pluginName, compilation => {
      const {
        mainTemplate
      } = compilation;
      mainTemplate.hooks.requireEnsure.tap(pluginName, (source, chunk, hash) => {
        return _webpack.Template.asString([source, '', '// I18n chunk loading logic', '(function() {', _webpack.Template.indent([
        // Initialize loading state tracking
        'if (!__webpack_require__.i18n) {', _webpack.Template.indent(['__webpack_require__.i18n = {', '  // Track loaded chunks by locale', '  loaded: {},', '', '  // Current and fallback manifests', '  manifest: null,', '  fallbackManifest: null,', '', '  // Load manifest with fallback support', '  loadManifest: function() {', '    var self = this;', '    if (this.manifest) return Promise.resolve(this.manifest);', '', '    return fetch(__webpack_require__.p + "i18n-manifest.json")', '      .then(function(res) {', '        if (!res.ok) throw new Error("No manifest");', '        return res.json();', '      })', '      .then(function(data) {', '        self.manifest = data;', '        return data;', '      })', '      .catch(function(error) {', '        return fetch(__webpack_require__.p + "i18n-manifest.main.json")', '          .then(function(res) { return res.json(); })', '          .then(function(data) {', '            self.fallbackManifest = data;', '            return self.manifest || data;', '          });', '      });', '  },', '', '  // Find which chunk contains a key', '  findChunkForKey: function(key) {', '    // Check current manifest first', '    if (this.manifest && this.manifest.chunks) {', '      for (var chunk in this.manifest.chunks) {', '        if (this.manifest.chunks[chunk].includes(key)) {', '          return chunk;', '        }', '      }', '    }', '', '    // Check key movement history', '    if (this.manifest && this.manifest.keyMovements && this.manifest.keyMovements[key]) {', '      return this.manifest.keyMovements[key].newChunk;', '    }', '', '    // Check fallback manifest', '    if (this.fallbackManifest && this.fallbackManifest.chunks) {', '      for (var chunk in this.fallbackManifest.chunks) {', '        if (this.fallbackManifest.chunks[chunk].includes(key)) {', '          return chunk;', '        }', '      }', '    }', '', '    return null;', '  },', '', '  // Get all keys for a chunk including moved keys', '  loadChunkKeys: function(chunkId) {', '    var keys = [];', '    if (this.manifest && this.manifest.chunks[chunkId]) {', '      keys = keys.concat(this.manifest.chunks[chunkId]);', '    }', '', '    // Add keys that were moved to this chunk', '    if (this.manifest && this.manifest.keyMovements) {', '      Object.entries(this.manifest.keyMovements)', '        .forEach(function([key, movement]) {', '          if (movement.newChunk === chunkId) {', '            keys.push(key);', '          }', '        });', '    }', '', '    return keys;', '  },', '', '  // Load a chunk for a specific locale', '  loadLocaleChunk: function(chunkId, locale) {', '    var self = this;', '    var loadedKey = chunkId + "_" + locale;', '', '    if (self.loaded[loadedKey]) {', '      return Promise.resolve();', '    }', '', '    return this.loadManifest().then(function(manifest) {', '      var chunkPath = manifest.locales[locale][chunkId];', '      if (!chunkPath) {', '        console.warn("No i18n chunk found for", {chunkId: chunkId, locale: locale});', '        return Promise.resolve();', '      }', '', '      return new Promise(function(resolve, reject) {', '        var script = document.createElement("script");', '        script.charset = "utf-8";', '        script.timeout = 120;', '        script.src = __webpack_require__.p + chunkPath;', '', '        var timeout = setTimeout(function() {', '          onScriptError({ type: "timeout", target: script });', '        }, 120000);', '', '        function onScriptComplete() {', '          script.onerror = script.onload = null;', '          clearTimeout(timeout);', '          self.loaded[loadedKey] = true;', '          resolve();', '        }', '', '        function onScriptError(event) {', '          script.onerror = script.onload = null;', '          clearTimeout(timeout);', '          var errorType = event && event.type;', '          var realSrc = event && event.target && event.target.src;', '          var error = new Error("Loading i18n chunk " + chunkId + " failed.\\n(" + errorType + ": " + realSrc + ")");', '          error.type = errorType;', '          error.request = realSrc;', '          reject(error);', '        }', '', '        script.onerror = onScriptError;', '        script.onload = onScriptComplete;', '', '        document.head.appendChild(script);', '      });', '    });', '  }', '};']), '}', '', '// Load i18n for the current chunk', 'var locale = ' + this.options.localeVarName + ';', 'if (locale && chunkId) {', _webpack.Template.indent(['promises.push(__webpack_require__.i18n.loadLocaleChunk(chunkId, locale));']), '}']), '})();']);
      });
    });
  }
}
exports.default = I18nDownloadLogic;