"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _webpackSources = require("webpack-sources");
var _path = _interopRequireDefault(require("path"));
var _fs = _interopRequireDefault(require("fs"));
function _interopRequireDefault(obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
}

// UnicodeConversionPlugin.js
class UnicodeConversionPlugin {
  constructor(options = {}) {
    this.options = {
      test: '',
      // i18nPublicPath
      encoding: 'utf8',
      debug: false,
      ...options
    };
  }
  apply(compiler) {
    // Run before the I18n plugin
    compiler.hooks.beforeRun.tapAsync('UnicodeConversionPlugin', (compiler, callback) => {
      try {
        const targetPath = _path.default.resolve(compiler.context, this.options.test);
        if (!_fs.default.existsSync(targetPath)) {
          throw new Error(`Path not found: ${targetPath}`);
        } // Convert files in place

        this.convertFilesInPlace(targetPath);
        callback();
      } catch (err) {
        callback(err);
      }
    });
  }
  convertFilesInPlace(targetPath) {
    const processDir = dir => {
      const files = _fs.default.readdirSync(dir);
      files.forEach(file => {
        const fullPath = _path.default.join(dir, file);
        const stat = _fs.default.statSync(fullPath);
        if (stat.isDirectory()) {
          processDir(fullPath);
        } else if (file.endsWith('.properties')) {
          try {
            // Read file
            const content = _fs.default.readFileSync(fullPath, this.options.encoding); // Convert content

            const convertedContent = this.convertUnicode(content); // Write back to same file

            _fs.default.writeFileSync(fullPath, convertedContent, this.options.encoding);
            if (this.options.debug) {
              console.log(`[UnicodeConversionPlugin] Converted: ${_path.default.relative(targetPath, fullPath)}`);
            }
          } catch (error) {
            console.error(`Error processing ${fullPath}: ${error.message}`);
          }
        }
      });
    };
    processDir(targetPath);
  }
  convertUnicode(content) {
    const lines = content.split(/\r?\n/);
    const converted = lines.map(line => {
      // Skip comments and empty lines
      if (line.startsWith('#') || !line.trim()) {
        return line;
      } // Find key-value separator

      const equalIndex = line.indexOf('=');
      if (equalIndex === -1) {
        return line;
      }
      const key = line.substring(0, equalIndex).trim();
      const value = line.substring(equalIndex + 1).trim(); // Convert unicode escapes in the value

      const convertedValue = this.convertUnicodeEscapes(value);
      return `${key}=${convertedValue}`;
    });
    return converted.join('\n');
  }
  convertUnicodeEscapes(str) {
    return str.replace(/\\u([0-9a-fA-F]{4})/g, (match, hex) => {
      try {
        return String.fromCharCode(parseInt(hex, 16));
      } catch (err) {
        throw new Error(`Invalid unicode escape sequence: ${match}`);
      }
    });
  }
}
var _default = UnicodeConversionPlugin;
exports.default = _default;