"use strict";

const esprima = require('esprima');
const crypto = require('crypto');
class I18nKeyHasher {
  constructor() {
    this.keyHashMap = new Map();
    this.hashKeyMap = new Map();
    this.usedHashes = new Set();
    this.hashLength = 5;
  }
  generateHash(key) {
    const fullHash = crypto.createHash('md5').update(key).digest('hex');
    let hash = fullHash.substring(0, this.hashLength);
    let counter = 0;
    let finalHash = hash;
    while (this.usedHashes.has(finalHash)) {
      counter++;
      finalHash = hash + counter;
    }
    this.usedHashes.add(finalHash);
    return finalHash;
  }
  hashKey(key) {
    if (this.keyHashMap.has(key)) {
      return this.keyHashMap.get(key);
    }
    const hash = this.generateHash(key);
    this.keyHashMap.set(key, hash);
    this.hashKeyMap.set(hash, key);
    return hash;
  }
  replaceKeysInSource(sourceCode) {
    try {
      // Parse source using esprima
      const ast = esprima.parse(sourceCode, {
        range: true,
        tokens: true,
        comment: true,
        jsx: true
      }); // Track string literals to replace

      const replacements = []; // Walk the AST to find string literals

      const traverse = node => {
        if (node.type === 'Literal' && typeof node.value === 'string' && this.keyHashMap.has(node.value)) {
          replacements.push({
            range: node.range,
            value: this.keyHashMap.get(node.value)
          });
        } // Traverse child nodes

        for (const key in node) {
          if (node[key] && typeof node[key] === 'object') {
            if (Array.isArray(node[key])) {
              node[key].forEach(child => child && traverse(child));
            } else {
              traverse(node[key]);
            }
          }
        }
      };
      traverse(ast); // Apply replacements in reverse order

      replacements.sort((a, b) => b.range[0] - a.range[0]).forEach(({
        range,
        value
      }) => {
        sourceCode = sourceCode.slice(0, range[0]) + `"${value}"` + sourceCode.slice(range[1]);
      });
      return sourceCode;
    } catch (err) {
      console.warn('Error during source transformation:', err);
      return sourceCode; // Return original source if transformation fails
    }
  }
}
module.exports = I18nKeyHasher;