"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getPropertiesAsJSON;
var _fs = require("fs");
function isComment(line) {
  return line.startsWith('#');
}
function getPropertiesAsJSON(filePath) {
  console.log(filePath);
  try {
    const data = (0, _fs.readFileSync)(filePath, 'utf8');
    const lines = data.split(/\r?\n/);
    const i18nObj = {};
    let currentSection = null;
    lines.forEach(fline => {
      const line = fline.trim();
      if (!line) {
        return;
      }
      if (isComment(line)) {
        // Check for section start or end
        if (/^#.*Starts/i.test(line)) {
          currentSection = line.replace(/^#\s*|\s*Starts.*$/gi, '').trim();
          if (!i18nObj[currentSection]) {
            i18nObj[currentSection] = {};
          }
        } else if (/^#.*Ends/i.test(line)) {
          currentSection = null;
        }
        return;
      }
      const ind = line.indexOf('=');
      if (ind > 0) {
        const key = line.slice(0, ind).trim();
        const value = line.slice(ind + 1).trim();
        if (currentSection) {
          if (!i18nObj[currentSection]) {
            i18nObj[currentSection] = {};
          }
          i18nObj[currentSection][key] = value;
        } else {
          i18nObj[key] = value;
        }
      }
    });
    return i18nObj;
  } catch (err) {
    console.error(`Error reading properties file at ${filePath}:`, err);
    return {};
  }
}